<?php

namespace Lc\PietroBundle\Definition\Field\IndividualData;

use EasyCorp\Bundle\EasyAdminBundle\Field\AssociationField;
use EasyCorp\Bundle\EasyAdminBundle\Field\EmailField;
use EasyCorp\Bundle\EasyAdminBundle\Field\HiddenField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;
use Lc\PietroBundle\Controller\IndividualData\IndividualDataAdminController;
use Lc\PietroBundle\Form\Dream\DreamType;
use Lc\PietroBundle\Form\ProjectBoost\ProjectBoostType;
use Lc\PietroBundle\Form\ProjectInspiring\ProjectInspiringType;
use Lc\PietroBundle\Form\Revolt\RevoltType;
use Lc\SovBundle\Definition\Field\AbstractFieldDefinition;
use Lc\SovBundle\Field\CollectionField;
use Lc\SovBundle\Field\StatusField;
use Lc\SovBundle\Translation\TranslatorAdmin;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;

class IndividualDataFieldDefinition extends AbstractFieldDefinition
{

    protected AuthorizationCheckerInterface $authorizationChecker;

    public function __construct(TranslatorAdmin $translatorAdmin, AuthorizationCheckerInterface $authorizationChecker)
    {
        $this->authorizationChecker = $authorizationChecker;

        parent::__construct($translatorAdmin);
    }

    public function configureFieldsIndex(): array
    {
        return [
            'firstname',
            'lastname',
            'territory',
            'nbDream',
            'nbRevolt',
            'nbProjectBoost',
            'nbProjectInspiring',
        ];
    }

    public function configurePanels(): array
    {
        return [];
    }

    public function configureFields(): array
    {
        $fields = array();

        $fields['firstname'] = TextField::new('firstname');
        $fields['lastname'] = TextField::new('lastname');
        $fields['email'] = EmailField::new('email');
        $fields['territory'] = AssociationField::new('territory')
            ->setTemplatePath('crud/field/association.html.twig');
        $fields['nbDream'] = TextField::new('nbDream')
            ->onlyOnIndex();
        $fields['nbRevolt'] = TextField::new('nbRevolt')
            ->onlyOnIndex();
        $fields['nbProjectBoost'] = TextField::new('nbProjectBoost')
            ->onlyOnIndex();
        $fields['nbProjectInspiring'] = TextField::new('nbProjectInspiring')
            ->onlyOnIndex();
        $fields['revolt'] = CollectionField::new('revolt')
            ->setFormTypeOption('entry_type', RevoltType::class)
            ->setFormTypeOption('by_reference', false)
            ->setRequired(false)
            ->hideOnIndex();
        $fields['dream'] = CollectionField::new('dream')
            ->setFormTypeOption('entry_type', DreamType::class)
            ->setFormTypeOption('by_reference', false)
            ->setRequired(false)
            ->hideOnIndex();
        $fields['projectBoost'] = CollectionField::new('projectBoost')
            ->setFormTypeOption('entry_type', ProjectBoostType::class)
            ->setFormTypeOption('by_reference', false)
            ->setRequired(false)
            ->hideOnIndex();
        $fields['projectinspiring'] = CollectionField::new('projectinspiring')
            ->setFormTypeOption('entry_type', ProjectInspiringType::class)
            ->setFormTypeOption('by_reference', false)
            ->setRequired(false)
            ->hideOnIndex();
        $hasAccess = $this->authorizationChecker->isGranted('ROLE_ADMIN');
        if ($hasAccess) {
            $fields['status'] = StatusField::new('status')
                ->setFormTypeOption('data', 0)
                ->setFormTypeOption('choices', ['Validé' => 1, 'En attente' => 0])
                ->setCustomOption('toggle_label', 'Valider')
                ->hideOnIndex();
        } else {
            $fields['status'] = HiddenField::new('status')
                ->setFormTypeOption('data', 0)
                ->hideOnIndex();
        }


        return $fields;
    }

}
